==> 0.8.0.tar.gz: test/model/field_editor_test.dart <==
@TestGroup(description: 'FieldEditor')
library fields.test.model.field_editor;

import 'package:bench/bench.dart';
import 'package:unittest/unittest.dart';
import 'package:fields/model.dart';

main() => reflectTests();

// -----------------------------------------------------------------------------
// Setup
// -----------------------------------------------------------------------------

@Setup
void setup() {  
  fieldEditors.addAll(defaultFieldEditors);
  fieldEditorFactories.addAll(defaultFieldEditorFactories);
}

@Teardown
void teardown() {
  fieldEditors.clear();
  fieldEditorFactories.clear();
}

class Rectangle {
  int x, y, w, h;
  Rectangle(this.x, this.y, this.w, this.h);
  String toString() => 'rectangle($x,$y,$w,$h)';
}

class HasFields {
  int i = 7;
  String s = 'snarf';
  Rectangle r = new Rectangle(1, 2, 3, 4);
}

class NotAnEditor { }

class IntEditor extends FieldEditor<int> {
  final element = null;
  IntEditor(Field field) : super(field);
  void refreshElement() {}
}

class StringEditor extends FieldEditor<String> {
  final element = null;
  StringEditor(Field field) : super(field);
  void refreshElement() {}
}

class RectangleEditor extends FieldEditor<Rectangle> {
  final element = null;
  RectangleEditor(Field field) : super(field);
  void refreshElement() {}
}

// -----------------------------------------------------------------------------
// Factory
// -----------------------------------------------------------------------------

@Test('Test the registration and use of a custom int editor.')
void testCustomIntEditor() {
  fieldEditors[int] = IntEditor;
  fieldEditorFactories[IntEditor] = (f) => new IntEditor(f);
  final obj = new HasFields();
  final i = new Field<int>('i', () => obj.i);
  final iEditor = new FieldEditor.forField(i);
  expect(iEditor, new isInstanceOf<IntEditor>());
  expect(iEditor.field, equals(i));
}

@Test('Test the registration and use of custom int and string editors.')
void testCustomIntAndStringEditor() {
  fieldEditors[int] = IntEditor;
  fieldEditors[String] = StringEditor;
  fieldEditorFactories[IntEditor] = (f) => new IntEditor(f);
  fieldEditorFactories[StringEditor] = (f) => new StringEditor(f);
  final obj = new HasFields();
  final i = new Field<int>('i', () => obj.i);
  final s = new Field<String>('s', () => obj.s);
  final iEditor = new FieldEditor.forField(i);
  final sEditor = new FieldEditor.forField(s);
  expect(iEditor, new isInstanceOf<IntEditor>());
  expect(iEditor.field, equals(i));
  expect(sEditor, new isInstanceOf<StringEditor>());
  expect(sEditor.field, equals(s));
}

// -----------------------------------------------------------------------------
// Value
// -----------------------------------------------------------------------------

@Test('Test getting the value of an int field in an int editor.')
void testIntFieldIntEditorGetValue() {
  fieldEditors[int] = IntEditor;
  fieldEditorFactories[IntEditor] = (f) => new IntEditor(f);
  final obj = new HasFields();
  final i = new Field<int>('i', () => obj.i, setter: (v) => obj.i = v);
  IntEditor editor = new FieldEditor.forField(i);
  expect(editor.isEnabled, isTrue);
  expect(editor.value, equals(7));
}

@Test('Test getting the value of an int field in a string editor.')
void testIntFieldStringEditorGetValue() {
  fieldEditors[int] = StringEditor;
  fieldEditorFactories[StringEditor] = (f) => new StringEditor(f);
  final obj = new HasFields();
  final i = new Field<int>('i', () => obj.i, setter: (v) => obj.i = v);
  StringEditor editor = new FieldEditor.forField(i);
  expect(editor.isEnabled, isTrue);
  expect(editor.value, equals('7'));
}

@Test('Test getting the value of a string field in an int editor throws.')
@ExpectError(const isInstanceOf<ValueTransformError>())
void testStringFieldIntEditorGetValueThrows() {
  fieldEditors[String] = IntEditor;
  fieldEditorFactories[IntEditor] = (f) => new IntEditor(f);
  final obj = new HasFields();
  final s = new Field<String>('s', () => obj.s);
  IntEditor editor = new FieldEditor.forField(s);
  // This should throw because the value 'snarf' cannot be parsed as an int.
  var v = editor.value;
}

@Test('Test that the value of a rectangle field in a string editor is '
      'read-only when no string -> rectangle transform is registered.')
void testRectagleFieldStringEditorValueReadOnly() {
  fieldEditors[Rectangle] = StringEditor;
  fieldEditorFactories[StringEditor] = (f) => new StringEditor(f);
  final obj = new HasFields();
  final r = new Field<Rectangle>('r', () => obj.r, setter: (v) => obj.r = v);
  StringEditor editor = new FieldEditor.forField(r);
  // There is no registered String -> Rectangle transform so we expect readonly.
  expect(editor.isEnabled, isFalse);
  expect(editor.value, equals('rectangle(1,2,3,4)'));
}

@Test('Test setting the value of an int field in an int editor.')
void testIntFieldIntEditorSetValue() {
  fieldEditors[int] = IntEditor;
  fieldEditorFactories[IntEditor] = (f) => new IntEditor(f);
  final obj = new HasFields();
  final i = new Field<int>('i', () => obj.i, setter: (v) => obj.i = v);
  IntEditor editor = new FieldEditor.forField(i);
  expect(editor.isEnabled, isTrue);
  editor.value = 42;
  expect(editor.value, equals(42));
  expect(obj.i, equals(42));
}

@Test('Test setting the value of an int field in a string editor.')
void testIntFieldStringEditorSetValue() {
  fieldEditors[int] = StringEditor;
  fieldEditorFactories[StringEditor] = (f) => new StringEditor(f);
  final obj = new HasFields();
  final i = new Field<int>('i', () => obj.i, setter: (v) => obj.i = v);
  StringEditor editor = new FieldEditor.forField(i);
  expect(editor.isEnabled, isTrue);
  editor.value = '42';
  expect(editor.value, equals('42'));
  expect(obj.i, equals(42));
}

@Test('Test setting bad value of an int field in a string editor throws.')
@ExpectError(const isInstanceOf<ValueTransformError>())
void testIntFieldStringEditorSetValueThrows() {
  fieldEditors[int] = StringEditor;
  fieldEditorFactories[StringEditor] = (f) => new StringEditor(f);
  final obj = new HasFields();
  final i = new Field<int>('i', () => obj.i, setter: (v) => obj.i = v);
  StringEditor editor = new FieldEditor.forField(i);
  expect(editor.isEnabled, isTrue);
  // This should throw because the value cannot be parsed as an int.
  editor.value = '42bottlesofbeeronthewall';
}

@Test('Test that setting the value of a rectangle field in a string '
      'editor throws because the editor is read-only.')
@ExpectError(isUnsupportedError)      
void testRectangleFieldStringEditorSetValueThrows() {
  fieldEditors[Rectangle] = StringEditor;
  fieldEditorFactories[StringEditor] = (f) => new StringEditor(f);
  final obj = new HasFields();
  final r = new Field<Rectangle>('r', () => obj.r, setter: (v) => obj.r = v);
  StringEditor editor = new FieldEditor.forField(r);
  // There is no registered String -> Rectangle transform so we expect readonly.
  expect(editor.isEnabled, isFalse);
  // Setting a value when read-only is expected to throw.
  editor.value = new Rectangle(5,6,7,8);
}

==> 0.11.0.tar.gz: test/model/field_editor_test.dart <==
@TestGroup('FieldEditor')
library fields.test.model.field_editor;

import 'package:bench/bench.dart';
import 'package:unittest/unittest.dart';
import 'package:fields/model.dart';

main() => reflectTests();

// -----------------------------------------------------------------------------
// Setup
// -----------------------------------------------------------------------------

@Setup
void setup() {  
  fieldEditors.addAll(defaultFieldEditors);
  fieldEditorFactories.addAll(defaultFieldEditorFactories);
}

@Teardown
void teardown() {
  fieldEditors.clear();
  fieldEditorFactories.clear();
}

class Rectangle {
  int x, y, w, h;
  Rectangle(this.x, this.y, this.w, this.h);
  String toString() => 'rectangle($x,$y,$w,$h)';
}

class HasFields {
  int i = 7;
  String s = 'snarf';
  Rectangle r = new Rectangle(1, 2, 3, 4);
}

class NotAnEditor { }

class IntEditor extends FieldEditor<int> {
  final element = null;
  IntEditor(Field field) : super(field);
  void refreshElement() {}
}

class StringEditor extends FieldEditor<String> {
  final element = null;
  StringEditor(Field field) : super(field);
  void refreshElement() {}
}

class RectangleEditor extends FieldEditor<Rectangle> {
  final element = null;
  RectangleEditor(Field field) : super(field);
  void refreshElement() {}
}

// -----------------------------------------------------------------------------
// Factory
// -----------------------------------------------------------------------------

@Test('Test the registration and use of a custom int editor.')
void testCustomIntEditor() {
  fieldEditors[int] = IntEditor;
  fieldEditorFactories[IntEditor] = (f) => new IntEditor(f);
  final obj = new HasFields();
  final i = new Field<int>('i', () => obj.i);
  final iEditor = new FieldEditor.forField(i);
  expect(iEditor, new isInstanceOf<IntEditor>());
  expect(iEditor.field, equals(i));
}

@Test('Test the registration and use of custom int and string editors.')
void testCustomIntAndStringEditor() {
  fieldEditors[int] = IntEditor;
  fieldEditors[String] = StringEditor;
  fieldEditorFactories[IntEditor] = (f) => new IntEditor(f);
  fieldEditorFactories[StringEditor] = (f) => new StringEditor(f);
  final obj = new HasFields();
  final i = new Field<int>('i', () => obj.i);
  final s = new Field<String>('s', () => obj.s);
  final iEditor = new FieldEditor.forField(i);
  final sEditor = new FieldEditor.forField(s);
  expect(iEditor, new isInstanceOf<IntEditor>());
  expect(iEditor.field, equals(i));
  expect(sEditor, new isInstanceOf<StringEditor>());
  expect(sEditor.field, equals(s));
}

// -----------------------------------------------------------------------------
// Value
// -----------------------------------------------------------------------------

@Test('Test getting the value of an int field in an int editor.')
void testIntFieldIntEditorGetValue() {
  fieldEditors[int] = IntEditor;
  fieldEditorFactories[IntEditor] = (f) => new IntEditor(f);
  final obj = new HasFields();
  final i = new Field<int>('i', () => obj.i, setter: (v) => obj.i = v);
  IntEditor editor = new FieldEditor.forField(i);
  expect(editor.isEnabled, isTrue);
  expect(editor.value, equals(7));
}

@Test('Test getting the value of an int field in a string editor.')
void testIntFieldStringEditorGetValue() {
  fieldEditors[int] = StringEditor;
  fieldEditorFactories[StringEditor] = (f) => new StringEditor(f);
  final obj = new HasFields();
  final i = new Field<int>('i', () => obj.i, setter: (v) => obj.i = v);
  StringEditor editor = new FieldEditor.forField(i);
  expect(editor.isEnabled, isTrue);
  expect(editor.value, equals('7'));
}

@Test('Test getting the value of a string field in an int editor throws.')
@ExpectError(isFormatException)
void testStringFieldIntEditorGetValueThrows() {
  fieldEditors[String] = IntEditor;
  fieldEditorFactories[IntEditor] = (f) => new IntEditor(f);
  final obj = new HasFields();
  final s = new Field<String>('s', () => obj.s);
  IntEditor editor = new FieldEditor.forField(s);
  // This should throw because the value 'snarf' cannot be parsed as an int.
  var v = editor.value;
}

@Test('Test that the value of a rectangle field in a string editor is '
      'read-only when no string -> rectangle transform is registered.')
void testRectagleFieldStringEditorValueReadOnly() {
  fieldEditors[Rectangle] = StringEditor;
  fieldEditorFactories[StringEditor] = (f) => new StringEditor(f);
  final obj = new HasFields();
  final r = new Field<Rectangle>('r', () => obj.r, setter: (v) => obj.r = v);
  StringEditor editor = new FieldEditor.forField(r);
  // There is no registered String -> Rectangle transform so we expect readonly.
  expect(editor.isEnabled, isFalse);
  expect(editor.value, equals('rectangle(1,2,3,4)'));
}

@Test('Test setting the value of an int field in an int editor.')
void testIntFieldIntEditorSetValue() {
  fieldEditors[int] = IntEditor;
  fieldEditorFactories[IntEditor] = (f) => new IntEditor(f);
  final obj = new HasFields();
  final i = new Field<int>('i', () => obj.i, setter: (v) => obj.i = v);
  IntEditor editor = new FieldEditor.forField(i);
  expect(editor.isEnabled, isTrue);
  editor.value = 42;
  expect(editor.value, equals(42));
  expect(obj.i, equals(42));
}

@Test('Test setting the value of an int field in a string editor.')
void testIntFieldStringEditorSetValue() {
  fieldEditors[int] = StringEditor;
  fieldEditorFactories[StringEditor] = (f) => new StringEditor(f);
  final obj = new HasFields();
  final i = new Field<int>('i', () => obj.i, setter: (v) => obj.i = v);
  StringEditor editor = new FieldEditor.forField(i);
  expect(editor.isEnabled, isTrue);
  editor.value = '42';
  expect(editor.value, equals('42'));
  expect(obj.i, equals(42));
}

@Test('Test setting bad value of an int field in a string editor throws.')
@ExpectError(isFormatException)
void testIntFieldStringEditorSetValueThrows() {
  fieldEditors[int] = StringEditor;
  fieldEditorFactories[StringEditor] = (f) => new StringEditor(f);
  final obj = new HasFields();
  final i = new Field<int>('i', () => obj.i, setter: (v) => obj.i = v);
  StringEditor editor = new FieldEditor.forField(i);
  expect(editor.isEnabled, isTrue);
  // This should throw because the value cannot be parsed as an int.
  editor.value = '42bottlesofbeeronthewall';
}

@Test('Test that setting the value of a rectangle field in a string '
      'editor throws because the editor is read-only.')
@ExpectError(isAssertionError)      
void testRectangleFieldStringEditorSetValueThrows() {
  fieldEditors[Rectangle] = StringEditor;
  fieldEditorFactories[StringEditor] = (f) => new StringEditor(f);
  final obj = new HasFields();
  final r = new Field<Rectangle>('r', () => obj.r, setter: (v) => obj.r = v);
  StringEditor editor = new FieldEditor.forField(r);
  // There is no registered String -> Rectangle transform so we expect readonly.
  expect(editor.isEnabled, isFalse);
  // Setting a value when read-only is expected to throw.
  editor.value = new Rectangle(5,6,7,8);
}
